import requests
import json
import time
from .classes import Interaction
from .utils import convert_to_string, URLS


def carterRequest(url, data, headers):
    try:
        response = requests.post(
            url, data=json.dumps(data), headers=headers)
        carter_data = response.json()
        return {"carter_data": carter_data, "status_code": response.status_code, "status_message": response.reason, "ok": response.ok}
    except (requests.exceptions.RequestException, json.JSONDecodeError) as e:
        return None


class Carter:
    def __init__(self, api_key):
        self.api_key = api_key
        self.history = []

    def say(self, text, player_id):
        text = convert_to_string("text", text)
        player_id = convert_to_string("player_id", player_id)
        start = time.perf_counter()
        data = {
            "text": text,
            "playerId": player_id,
            "key": self.api_key,
        }

        headersList = {
            "Accept": "*/*",
            "Content-Type": "application/json"
        }

        response = carterRequest(URLS["say"], data, headers=headersList)
        time_taken = int((time.perf_counter() - start) * 1000)
        interaction = Interaction("say", data, response, time_taken)
        if interaction.ok:
            self.history.insert(0, interaction)
        return interaction

    def opener(self, player_id):
        player_id = convert_to_string("player_id", player_id)
        start = time.perf_counter()
        data = {
            "playerId": player_id,
            "key": self.api_key,
        }
        headersList = {
            "Accept": "*/*",
            "Content-Type": "application/json"
        }
        response = carterRequest(
            URLS["opener"], headers=headersList, data=data)
        time_taken = int((time.perf_counter() - start) * 1000)
        interaction = Interaction("opener", data, response, time_taken)
        if interaction.ok:
            self.history.insert(0, interaction)
        return interaction

    def personalise(self, text):
        text = convert_to_string("text", text)
        start = time.perf_counter()
        data = {
            "text": text,
            "key": self.api_key,
        }
        headersList = {
            "Accept": "*/*",
            "Content-Type": "application/json"
        }
        response = carterRequest(
            URLS["personalise"], headers=headersList, data=data)
        time_taken = int((time.perf_counter() - start) * 1000)
        interaction = Interaction("personalise", data, response, time_taken)
        return interaction



import uuid
from .utils import iso_now
import uuid
from .utils import iso_now


class Interaction():
    def __init__(self, type, payload, response_data, time_taken):

        # set easy stuff
        self.type = type
        self.id = uuid.uuid1()
        self.payload = payload
        self.time_taken = time_taken
        self.timestamp = iso_now()
        self.triggered_skills = []
        self.executed_skills = []

        # set response params
        if response_data is not None:
            self.ok = response_data["ok"]
            self.status_code = response_data["status_code"]
            self.status_message = response_data["status_message"]
            carter_data = response_data["carter_data"]
            if carter_data is not None:
                if "output" in carter_data:
                    self.output_text = carter_data["output"]["text"]
                    self.output_audio = carter_data["output"]["audio"]
                else:
                    self.output_text = None
                    self.output_audio = None

                if "content" in carter_data:
                    self.output_text = carter_data["content"]
                if "sentence" in carter_data:
                    self.output_text = carter_data["sentence"]
                if "output_text" not in self.__dict__:
                    self.output_text = None


                if "forced_behaviours" in carter_data:
                    self.forced_behaviours = carter_data["forced_behaviours"]
                else:
                    self.forced_behaviours = None
            else:
                self.ok = False
                self.output_text = None
                self.forced_behaviours = None

        else:
            self.ok = False
            self.status_code = None
            self.status_message = None
            self.output_text = None
            self.forced_behaviours = None

        # Payload data
        if "text" in self.payload:
            self.input_text = self.payload["text"]
        else:
            self.input_text = self.type

        if "player_id" in self.payload:
            self.player_id = self.payload["player_id"]
        else:
            self.player_id = None

    def __str__(self):
        if self.ok:
            return f"{self.type.capitalize()} interaction {self.id} - {self.input_text} -> {self.output_text}"
        else:
            return f"{self.type.capitalize()} interaction {self.id} - Failed with status code {self.status_code} and message {self.status_message}"

    def print_verbose(self):
        for key, value in self.__dict__.items():
            print("=====================================")
            print(f"{self.type.upper()} INTERACTION {self.id}")
            print(f"{key}: {value}")
            print("=====================================")



import datetime

URLS = {
    "say": "https://api.carterlabs.ai/chat",
    "opener": "https://api.carterlabs.ai/opener",
    "personalise": "https://api.carterlabs.ai/personalise",
}

def convert_to_string(variable_name, value):
    try:
        return str(value)
    except:
        raise TypeError(f"{variable_name} must be convertible to a string, received {type(value)}")

def iso_now():
    return datetime.datetime.now().isoformat()



from setuptools import setup, find_packages

setup(
    name="carter-py",
    version="0.1",
    packages=find_packages(),
    install_requires = [
    "aiohttp==3.8.4",
    "aioresponses==0.7.4",
    "asynctest==0.13.0",
    "python-dotenv==1.0.0",
    "Requests==2.28.2",
    "responses==0.23.1",
    "setuptools==65.5.0"
    ],
    author="LazyLyrics",
    author_email="lazylyrics@icloud.com",
    description="A wrapper for the Carter API",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/LazyLyrics/carterpy",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)
